%---------------------------------------------------
% 
%             %%%%%%%%%%%%%%
%             % iniciacHMM %
%             %%%%%%%%%%%%%%
%
% 
% This function calculates the CHMM optimal initial model for the Baum-Welch training.
% function [Aa,Ba,Meda,Vara,Pia]= iniciachmm(Ne,Np,BAKIS,salto,nftrain,lrep,agrup,Ngauss,maxitermi)
% 
% 
% Entry: 	Ne state number 
% Np number parameters by symbol
% 	if Bakis = 1 implements a Bakis HMM (also called left-right), else an ergodic HMM is defined 
% Salto is the maximum path authorized in the Bakis HMM from a state to another. 
% Nftrain: is the name of the file with the training data set 
% Lrep is a Matrix with the size from the training set data for every class, group and parameter (see also the function formato_lectura_secuencial)
% agrup defines how to cluster the parameters together
% Ngauss is the number of Gaussians used in the mixture of Gaussians.
% Maxitermi is the maximum iteration to calculate the initial HMM.
% 
% Result: 	Aa, Ba, Meda, Vara and Pia are the matrices for the parameters of the HMM. Meda and Vara are the mean and the variance for each Gaussian.
% 
% In this function, we will generate a number of symbols in every state bigger than twice Ns. Thus, we make sure to initialize correctly the HMM.
% 		
% We use the viterbic and the genchmm functions. And the netlab utility (gmm gmminit and gmmem).
% 
%---------------------------------------------------
function [Aa,Ba,Meda,Vara,Pia]=iniciachmm(Ne,Np,BAKIS,salto,nftrain,lrep,agrup,Ngauss,maxitermi)

% Ir reads the database for the training of this class and group 
% to initialize the initial model
fidvtrain=fopen(nftrain,'r');
dimrep=agrup(end)-agrup(1);
muestras=fread(fidvtrain,sum(lrep)*dimrep,'double');
muestras=reshape(muestras,sum(lrep),dimrep);
fclose(fidvtrain);
% we calculate the variables
vmean=cell(Np,1);vstd=cell(Np,1);vcombl=cell(Np,1);
for ip=1:Np
    mix = gmm(agrup(ip+1)-agrup(ip),Ngauss(ip),'spherical');
    options=foptions;
    options(1)=0;		% Prints out error values.
    options(5)=1;        %check variance
    options(14) = 10;
    mix = gmminit(mix,muestras(:,agrup(ip):agrup(ip+1)-1), options);
    mix = gmmem(mix, muestras(:,agrup(ip):agrup(ip+1)-1), options);
    vmean{ip}=mix.centres;
    vstd{ip}=mix.covars'*ones(1,agrup(ip+1)-agrup(ip))+1e-1;
    vcombl{ip}=mix.priors';                      
end
clear muestras

% It initializes the Markov models.
itermi=1;
% Initialization of the matrices of the Markov model
[A,B,Med,Var,Pi]=genchmm(Ne,vcombl,vmean,vstd,agrup,Ngauss,Np,BAKIS,salto);
Aa=A;Ba=B;Meda=Med;Vara=Var;Pia=Pi;

if gt(maxitermi,1)
   % To check that initialization is good, the critarion
   % is that all the states have more than twice symbols 
   % generated than Ns.
   % Calcul of the number of symbols generated by each state of the HMM.
   fidvtrain=fopen(nftrain,'r');
   nr=size(lrep,1);
   vlt=cell(1,1);
   for ir=1:nr,
       vlt{1}=fread(fidvtrain,lrep(ir)*dimrep,'double');
       vlt{1}=reshape(vlt{1},lrep(ir),dimrep);
       OqP{ir}=viterbic(A,B,Med,Var,Pi,vlt{1},agrup);
   end;
   fclose(fidvtrain);

   % And we calculate the largest distance to the equi-occupation
   dmim=max(abs(sum(lrep)/Ne-hist(cat(1,OqP{:}),Ne)));
   % If all the states do not have the number bigger than
   %  the minimum, we reionitiate the model.
   % Nota Bene: we can be blocked in an infinite loop
   while and(dmim>0.3*sum(lrep)/Ne,itermi<maxitermi),
      itermi=itermi+1;
      [A,B,Med,Var,Pi]=genchmm(Ne,vcombl,vmean,vstd,agrup,Ngauss,Np,BAKIS,salto);
      fidvtrain=fopen(nftrain,'r');
      nr=size(lrep,1);
      vlt=cell(1,1);
      for ir=1:nr,
          vlt{1}=fread(fidvtrain,lrep(ir)*dimrep,'double');
          vlt{1}=reshape(vlt{1},lrep(ir),dimrep);
          OqP{ir}=viterbic(A,B,Med,Var,Pi,vlt{1},agrup);
      end;
      fclose(fidvtrain);
      dmi=max(abs(sum(lrep)/Ne-hist(cat(1,OqP{:}),Ne)));
      if dmi<dmim,
         dmim=dmi;
         Aa=A;Ba=B;Pia=Pi;
      end;
   end;
end
return